@@ -41,6 +41,8 @@
 #include <sys/types.h>
 #include <sys/wait.h>
 
+#define DESKTOP_ENTRY_GROUP                "Desktop Entry"
+
 static gboolean
 recursive_mkdir (const char *path)
 {
@@ -256,7 +258,7 @@ eel_mime_add_application (const char *mime_type,
 		command_line, needs_terminal ? "true" : "false");
 
 	if (write_desktop_file (filename, desktop_text)) {
-		ret = gnome_vfs_mime_application_new_from_id (basename);
+		ret = gnome_vfs_mime_application_new_from_desktop_id (basename);
 		run_update_command ("update-desktop-database", "applications");
 	} else {
 		ret = NULL;
@@ -270,6 +272,87 @@ eel_mime_add_application (const char *mime_type,
 	return ret;
 }
 
+GnomeVFSMimeApplication *
+eel_mime_add_custom_mime_type_for_desktop_file (const char *desktop_file,
+						const char *mime_type)
+{
+	GnomeVFSMimeApplication *ret;
+	GKeyFile *keyfile;
+	char *application_dir;
+	char *filename;
+	char *basename, *new_basename;
+	char *extension;
+	int i;
+
+	if (desktop_file == NULL) {
+		return NULL;
+	}
+
+	if (mime_type == NULL) {
+		return NULL;
+	}
+	
+	if (!ensure_application_dir ()) {
+		return NULL;
+	}
+
+	application_dir = get_user_dir ("applications");
+
+	/* Get a filename */
+	basename = g_path_get_basename (desktop_file);
+	/* Strip off extension */
+	extension = strrchr (basename, '.');
+	if (extension != NULL) {
+		*extension = 0;
+	}
+
+	new_basename = g_strdup_printf ("%s-usercustom.desktop", basename);
+	filename = g_build_filename (application_dir, new_basename, NULL);
+
+	i = 1;
+	while (g_file_test (filename, G_FILE_TEST_EXISTS)) {
+		g_free (new_basename);
+		g_free (filename);
+		new_basename = g_strdup_printf ("%s-usercustom-%d.desktop", basename, i++);
+		filename = g_build_filename (application_dir, new_basename, NULL);
+	}
+
+	g_free (basename);
+	
+	keyfile = g_key_file_new ();
+	if (g_key_file_load_from_file (keyfile, desktop_file,
+				       G_KEY_FILE_KEEP_COMMENTS |
+				       G_KEY_FILE_KEEP_TRANSLATIONS, NULL)) {
+		char *desktop_text;
+		char *new_mimetypes;
+
+		new_mimetypes = g_strconcat (mime_type, ";", NULL);
+		g_key_file_set_string (keyfile, DESKTOP_ENTRY_GROUP, "MimeType", new_mimetypes);
+		g_free (new_mimetypes);
+		g_key_file_set_boolean (keyfile, DESKTOP_ENTRY_GROUP, "NoDisplay", TRUE);
+		desktop_text = g_key_file_to_data (keyfile, NULL, NULL);
+
+		if (write_desktop_file (filename, desktop_text)) {
+			ret = gnome_vfs_mime_application_new_from_desktop_id (new_basename);
+			run_update_command ("update-desktop-database", "applications");
+		} else {
+			ret = NULL;
+		}
+        	g_free (desktop_text);
+	} else {
+		ret = NULL;
+	}
+	
+	g_key_file_free (keyfile);
+	g_free (new_basename);
+	g_free (filename);
+	g_free (application_dir);
+	
+	return ret;
+}
+
+
+
 static char *
 get_override_filename (void)
 {
@@ -697,6 +780,39 @@ arg_is_exec_param (const char *param)
 	return FALSE;
 }
 
+GnomeVFSMimeApplication *
+eel_mime_check_for_desktop_duplicates (const char *mime_type,
+				       const char *desktop_file)
+{
+	GList *applications;
+	GList *list;
+
+	if (mime_type == NULL) {
+		return NULL;
+	}
+	
+	applications = gnome_vfs_mime_get_all_applications (mime_type);
+	if (applications == NULL) {
+		return NULL;
+	}
+
+	for (list = applications; list; list = list->next) {
+		GnomeVFSMimeApplication *application;
+		const char *path;
+		
+		application = list->data;
+		path = gnome_vfs_mime_application_get_desktop_file_path (application);
+		if (strcmp (desktop_file, path) == 0) {
+			g_list_free (applications);
+			return gnome_vfs_mime_application_copy (application);
+		}
+	}
+	g_list_free (applications);
+
+	return NULL;
+}
+
+
 GnomeVFSMimeApplication *
 eel_mime_check_for_duplicates (const char *mime_type,
 			       const char *command_line)
@@ -743,7 +859,7 @@ eel_mime_check_for_duplicates (const char *mime_type,
 		application = list->data;
 		/* We do a parsing of the application command and see if it's
 		 * identical to the command being passed in. */
-		g_shell_parse_argv (application->command,
+		g_shell_parse_argv (gnome_vfs_mime_application_get_exec (application),
 				    &app_argc, &app_argv, NULL);
 		if (app_argv == NULL) {
 			continue;
